@@ -14,16 +14,24 @@ from unittest import TestCase
 
 import pytest
 
-from hypothesis import assume, given, strategies as st
+from hypothesis import assume, given, settings, strategies as st
 from hypothesis.internal.compat import PYPY
 
 from tests.common.utils import skipif_emscripten
 
+pytestmark = pytest.mark.skipif(
+    settings._current_profile == "threading",
+    reason="bad interactions when mixing threads and asyncio",
+)
+
 
 def coro_decorator(f):
     with warnings.catch_warnings():
         warnings.simplefilter(action="ignore", category=DeprecationWarning)
-        return asyncio.coroutine(f)
+        try:
+            return asyncio.coroutine(f)
+        except AttributeError:
+            pytest.skip("needs fixing for asyncio version", allow_module_level=True)
 
 
 class TestAsyncio(TestCase):
